@@ -1,12 +1,11 @@
 import requests
 from .._utils.consts import SB_PROXY_CHATMEDIA, S_REDDIT, USER_AGENT, SB_User_Agent, SB_ai, WEB_USERAGENT, WWW_REDDIT
 import json
-from .models import Channel, Message
+from .models import Channel, Message, Members
 
 
 def _get_user_id(username):
-    response = requests.get(f"{WWW_REDDIT}/user/{username}/about.json",
-                            headers={'user-agent': WEB_USERAGENT}).json()
+    response = requests.get(f"{WWW_REDDIT}/user/{username}/about.json", headers={'user-agent': WEB_USERAGENT}).json()
     u_id = response.get('data', {}).get('id')
     if u_id is None:
         return None
@@ -16,13 +15,13 @@ def _get_user_id(username):
 
 class Tools:
     def __init__(self, reddit_auth):
+        self._reddit_auth = reddit_auth
         self._req_sesh = requests.Session()
         self._req_sesh.headers = {
             'User-Agent': USER_AGENT,
             'SendBird': f'Android,29,3.0.82,{SB_ai}',
             'SB-User-Agent': SB_User_Agent
         }
-        self._reddit_auth = reddit_auth
         try:
             _ = self._reddit_auth.reddit_username
             self._is_reauthable = True
@@ -45,17 +44,17 @@ class Tools:
 
     def rename_channel(self, name, channel_url, session_key):
         data = json.dumps({
-            # 'cover_url': '',
             'name': name,
-            # 'data': ''
         })
-        uri = f"{SB_PROXY_CHATMEDIA}/v3/group_channels/{channel_url}"
-        response = self._handled_req(method='PUT', uri=uri, data=data, headers={'Session-Key': session_key})
+        response = self._handled_req(method='PUT', uri=f"{SB_PROXY_CHATMEDIA}/v3/group_channels/{channel_url}",
+                                     headers={'Session-Key': session_key},
+                                     data=data)
         return Channel.from_dict(response.json())
 
     def delete_message(self, channel_url, msg_id, session_key):
-        uri = f"{SB_PROXY_CHATMEDIA}/v3/group_channels/{channel_url}/messages/{msg_id}"
-        self._handled_req(method='DELETE', uri=uri, headers={'Session-Key': session_key})
+        self._handled_req(method='DELETE',
+                          uri=f"{SB_PROXY_CHATMEDIA}/v3/group_channels/{channel_url}/messages/{msg_id}",
+                          headers={'Session-Key': session_key})
 
     def kick_user(self, channel_url, user_id, duration):
         data = json.dumps({
@@ -63,8 +62,8 @@ class Tools:
             'user_id': user_id,
             'duration': duration
         })
-        uri = f'{S_REDDIT}/api/v1/channel/kick/user'
-        self._handled_req(method='POST', uri=uri, headers={'Authorization': f'Bearer {self._reddit_auth.api_token}'},
+        self._handled_req(method='POST', uri=f'{S_REDDIT}/api/v1/channel/kick/user',
+                          headers={'Authorization': f'Bearer {self._reddit_auth.api_token}'},
                           data=data)
 
     def invite_user(self, channel_url, nicknames):
@@ -75,16 +74,16 @@ class Tools:
         data = json.dumps({
             'users': users
         })
-        url = f'{S_REDDIT}/api/v1/sendbird/group_channels/{channel_url}/invite'
-        self._handled_req(method='POST', uri=url, headers={'Authorization': f'Bearer {self._reddit_auth.api_token}'},
+        self._handled_req(method='POST', uri=f'{S_REDDIT}/api/v1/sendbird/group_channels/{channel_url}/invite',
+                          headers={'Authorization': f'Bearer {self._reddit_auth.api_token}'},
                           data=data)
 
     def accept_chat_invite(self, channel_url, session_key):
         data = json.dumps({
             'user_id': self._reddit_auth.user_id
         })
-        url = f'{SB_PROXY_CHATMEDIA}/v3/group_channels/{channel_url}/accept'
-        self._handled_req(method='PUT', uri=url, headers={'Session-Key': session_key}, data=data)
+        self._handled_req(method='PUT', uri=f'{SB_PROXY_CHATMEDIA}/v3/group_channels/{channel_url}/accept',
+                          headers={'Session-Key': session_key}, data=data)
 
     def get_channels(self, limit, order, show_member, show_read_receipt, show_empty, member_state_filter, super_mode,
                      public_mode, unread_filter, hidden_mode, show_frozen, session_key):
@@ -102,16 +101,32 @@ class Tools:
             'hidden_mode': hidden_mode,
             'show_frozen': show_frozen,
         }
-        url = f'{SB_PROXY_CHATMEDIA}/v3/users/{self._reddit_auth.user_id}/my_group_channels'
-        response = self._handled_req(method='GET', uri=url, headers={'Session-Key': session_key}, params=params)
+        response = self._handled_req(method='GET',
+                                     uri=f'{SB_PROXY_CHATMEDIA}/v3/users/{self._reddit_auth.user_id}/my_group_channels',
+                                     headers={'Session-Key': session_key}, params=params)
         return [Channel.from_dict(channel) for channel in response.json()['channels']]
 
+    def get_members(self, channel_url, next_token, limit, order, member_state_filter, session_key):
+        params = {
+            'token': next_token,
+            'limit': limit,
+            'order': order,
+            'muted_member_filter': 'all',
+            'member_state_filter': member_state_filter,
+            'show_member_is_muted': 'true',
+            'show_read_receipt': 'true',
+            'show_delivery_receipt': 'true',
+        }
+        response = self._handled_req(method='GET', uri=f'{SB_PROXY_CHATMEDIA}/v3/group_channels/{channel_url}/members',
+                                     headers={'Session-Key': session_key}, params=params)
+        return Members.from_dict(response.json())
+
     def leave_chat(self, channel_url, session_key):
         data = json.dumps({
             'user_id': self._reddit_auth.user_id
         })
-        url = f'{SB_PROXY_CHATMEDIA}/v3/group_channels/{channel_url}/leave'
-        self._handled_req(method='PUT', uri=url, headers={'Session-Key': session_key}, data=data)
+        self._handled_req(method='PUT', uri=f'{SB_PROXY_CHATMEDIA}/v3/group_channels/{channel_url}/leave',
+                          headers={'Session-Key': session_key}, data=data)
 
     def create_channel(self, nicknames, group_name, own_name):
         users = [{"user_id": self._reddit_auth.user_id, "nickname": own_name}]
@@ -121,8 +136,7 @@ class Tools:
             'users': users,
             'name': group_name
         })
-        url = f'{S_REDDIT}/api/v1/sendbird/group_channels'
-        response = self._handled_req(method='POST', uri=url,
+        response = self._handled_req(method='POST', uri=f'{S_REDDIT}/api/v1/sendbird/group_channels',
                                      headers={'Authorization': f'Bearer {self._reddit_auth.api_token}'},
                                      data=data)
         return Channel.from_dict(response.json())
@@ -136,14 +150,14 @@ class Tools:
         self._handled_req(method='PUT', uri=f'{SB_PROXY_CHATMEDIA}/v3/group_channels/{channel_url}/hide',
                           headers={'Session-Key': session_key}, data=data)
 
-    def get_older_messages(self, channel_url, prev_limit, next_limit, reverse, session_key):
+    def get_older_messages(self, channel_url, message_ts, prev_limit, reverse, session_key):
         params = {
             'is_sdk': 'true',
             'prev_limit': prev_limit,
-            'next_limit': next_limit,
+            'next_limit': '0',
             'include': 'false',
             'reverse': reverse,
-            'message_ts': 9007199254740991,
+            'message_ts': message_ts,
             'with_sorted_meta_array': 'false',
             'include_reactions': 'false',
             'include_thread_info': 'false',
